#ifndef MKF_BOOK_HPP_INCLUDED
#define MKF_BOOK_HPP_INCLUDED


#include"mkf_node.hpp"
#include"mkf_cursor.hpp"
#include"mkf_ascii.hpp"
#include"mkf_ctype.hpp"


namespace mkf{


using DefinitionList = std::list<Definition>;


struct Block;


struct
CompareContext
{
  const char*&       p0;
  const char*  const p1;

  Node*  nd;

  CompareContext(const char*&  p0_, const char*  p1_, Node*  nd_=nullptr):
  p0(p0_),
  p1(p1_),
  nd(nd_){}

};


class
Book
{
  Definition  main_definition;

  DefinitionList  sub_definitions;


  bool  compare_definition_reference(CompareContext&  ctx, const Definition&  def) const;

  bool  compare_for_any(CompareContext&  ctx, const Definition&  def) const;
  bool  compare_for_all(CompareContext&  ctx, const Definition&  def) const;

  bool  compare(CompareContext&  ctx, const Element&  elm) const;
  bool  compare(CompareContext&  ctx, const Definition&  def) const;


  void  read_definition(char*  buf, size_t  n, Definition&  def, const Node&  base);

  void  read_identifier(char*  buf, size_t  n, const Node&  base);

  void  read_entry_chain(char*  buf, size_t  n, Definition&  def, const Node&  base);
  void  read_entry(char*  buf, size_t  n, Definition&  def, const Node&  base);

  void  read_element(char*  buf, size_t  n, Definition&  def, uint32_t  flags, const Node&  base);

  void  read_mark(uint32_t&  flags, const Node&  base);

  void  read_literal_character(int&  c, const Node&  base);
  void  read_character(int&  c, const Node&  base);


public:


  void  change_main_definition(Definition&&  def);

  void  append_sub_definition(Definition&&  def);

  void  make(const std::string&  s);
  void  make(const Node&  src);


  void  clear();

  const Definition*  find(const Identifier&  id) const;

  Node     parse(const std::string&  s) const;
  Node  subparse(const std::string&  s) const;

  void  print() const;
  void  print(Printer&  pr) const;

};


}


#endif




