#include"mkf_book.hpp"
#include"mkf_identifier.hpp"


namespace mkf{


namespace{


bool
compare_ctype(CompareContext&  ctx, CType  type)
{
    if(test_ctype_code(*ctx.p0,type))
    {

      int  c = *ctx.p0++;

        if(ctx.nd)
        {
          ctx.nd->character = c;
        }


      return true;
    }


  return false;
}


bool
compare_ascii(CompareContext&  ctx, int  c)
{
    if(c == *ctx.p0)
    {
      ++ctx.p0;

        if(ctx.nd)
        {
          ctx.nd->character = c;
        }


      return true;
    }


  return false;
}


}


bool
Book::
compare_definition_reference(CompareContext&  ctx, const Definition&  def) const
{
  auto  p = ctx.p0;

  CompareContext  subctx(p,ctx.p1,new Node(def.get_identifier().get_buffer()));

    if(compare(subctx,def))
    {
      ctx.p0 = p;

      ctx.nd->append(subctx.nd);


      return true;
    }


  delete subctx.nd;

  return false;
}


bool
Book::
compare(CompareContext&  ctx, const Element&  elm) const
{
    switch(elm.get_kind())
    {
      case(ElementKind::null):
        report;
        break;
      case(ElementKind::character):
//printf("%c ?= %c;\n",elm.get_character(),*ctx.p0);
          if(elm.get_character() == *ctx.p0)
          {
            int  c = *ctx.p0++;

              if(ctx.nd)
              {
                ctx.nd->character = c;
              }


            return true;
          }
        break;
      case(ElementKind::identifier):
        {
          auto  defname = elm.get_identifier().get_buffer();

          auto  ctype = get_ctype(defname);

            if(ctype != CType::null)
            {
              return compare_ctype(ctx,ctype);
            }


          auto  c = find_ascii_code(defname);

            if(c)
            {
              return compare_ascii(ctx,c);
            }


          auto  def = find(elm.get_identifier());

            if(def)
            {
              return compare_definition_reference(ctx,*def);
            }

          else
            {
              report;
              printf("definition [%s] is not found.\n",defname);
              printf("%s.\n",ctx.p0);
            }
        }
        break;
      default:
        report;
    }


  return false;
}


bool
Book::
compare(CompareContext&  ctx, const Definition&  def) const
{
    switch(def.get_separator())
    {
      case('|'):
          if(!compare_for_any(ctx,def))
          {
            return false;
          }
        break;
      case(0):
      case(','):
      case(';'):
          if(!compare_for_all(ctx,def))
          {
            return false;
          }
        break;
      default:;
        report;
        return false;
    }


  return true;
}




bool
Book::
compare_for_any(CompareContext&  ctx, const Definition&  def) const
{
    for(auto&  elm: def.get_list())
    {
        if(!*ctx.p0 || (ctx.p0 >= ctx.p1))
        {
          return false;
        }


        if(compare(ctx,elm))
        {
            if(elm.test_repeat())
            {
                for(;;)
                {
                    if(!compare(ctx,elm))
                    {
                      break;
                    }
                }
            }


          return true;
        }
    }


  return false;
}


bool
Book::
compare_for_all(CompareContext&  ctx, const Definition&  def) const
{
    for(auto&  elm: def.get_list())
    {
        if(!*ctx.p0 || (ctx.p0 >= ctx.p1))
        {
          return true;
        }


        if(!compare(ctx,elm))
        {
            if(elm.test_omittable())
            {
              continue;
            }


          return false;
        }


          if(elm.test_repeat())
          {
              for(;;)
              {
                  if(!compare(ctx,elm))
                  {
                    break;
                  }
              }
          }
    }


  return true;
}



}




