#include"mkf_book.hpp"
#include"mkf_cursor.hpp"
#include"mkf_identifier.hpp"
#include<cstdlib>




namespace mkf{


void
Book::
make(const Node&  src)
{
  char  buf[256];

  Cursor  cur(src);

  DefinitionList  defls;

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd == "definition")
        {
          defls.emplace_back();

          auto&  def = defls.back();

          read_definition(buf,sizeof(buf),def,nd);
        }


      cur.advance();
    }


  main_definition = std::move(defls.front());

  defls.pop_front();

  sub_definitions = std::move(defls);
}


void
Book::
read_definition(char*  buf, size_t  n, Definition&  def, const Node&  base)
{
  Cursor  cur(base);

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd == "identifier")
        {
          read_identifier(buf,n,nd);

          def.change_identifier(buf);
        }

      else
        if(nd == "entry_chain")
        {
          read_entry_chain(buf,n,def,nd);
        }


      cur.advance();
    }
}


void
Book::
read_identifier(char*  buf, size_t  n, const Node&  base)
{
  base.scan_string(buf,n);
}


void
Book::
read_entry_chain(char*  buf, size_t  n, Definition&  def, const Node&  base)
{
  Cursor  cur(base);

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if((nd == "entry") ||
           (nd == "last_entry"))
        {
          read_entry(buf,n,def,nd);
        }


      cur.advance();
    }
}


void
Book::
read_entry(char*  buf, size_t  n, Definition&  def, const Node&  base)
{
  Cursor  cur(base);

  uint32_t  flags = 0;

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd == "mark")
        {
          flags = 0;

          read_mark(flags,nd);
        }

      else
        if(nd == "element")
        {
          read_element(buf,n,def,flags,nd);

          flags = 0;
        }

      else
        if(nd == "separator")
        {
          def.change_separator(nd.character);
        }

      else
        if(nd == "terminator")
        {
          break;
        }


      cur.advance();
    }
}


void
Book::
read_element(char*  buf, size_t  n, Definition&  def, uint32_t  flags, const Node&  base)
{
  Cursor  cur(base);

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd == "identifier")
        {
          read_identifier(buf,n,nd);

          def.append_element(Element(flags,buf));
        }

      else
        if(nd == "character_literal")
        {
          int  c;

          read_literal_character(c,nd);

          def.append_element(Element(flags,c));
        }


      cur.advance();
    }
}


void
Book::
read_literal_character(int&  c, const Node&  base)
{
  Cursor  cur(base);

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd == "character")
        {
          read_character(c,nd);

          break;
        }


      cur.advance();
    }
}


void
Book::
read_character(int&  c, const Node&  base)
{
  Cursor  cur(base);

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd == "null_character")
        {
          c = 0;

          break;
        }

      else
        if(nd == "newline_character")
        {
          c = '\n';

          break;
        }

      else
        if(nd == "backslash_character")
        {
          c = '\\';

          break;
        }

      else
        if(nd == "single_quote_character")
        {
          c = '\'';

          break;
        }

      else
        if(nd == "double_quote_character")
        {
          c = '\"';

          break;
        }

      else
        if(nd == "normal_character")
        {
          c = nd.character;

          break;
        }


      cur.advance();
    }
}


void
Book::
read_mark(uint32_t&  flags, const Node&  base)
{
  Cursor  cur(base);

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd.character == '*')
        {
          flags |= ElementFlag::repeat;
        }

      else
        if(nd.character == '?')
        {
          flags |= ElementFlag::omittable;
        }


      cur.advance();
    }
}


}



