#include"mkf_definition.hpp"
#include"mkf_ascii.hpp"
#include"mkf_ctype.hpp"




namespace mkf{




Definition::
Definition(const char*  id):
identifier(id),
separator(0)
{
}


Definition::
Definition(const Definition&   rhs)
{
  *this = rhs;
}


Definition::
Definition(Definition&&  rhs)
{
  *this = std::move(rhs);
}




Definition&
Definition::
operator=(const Definition&   rhs)
{
  identifier = rhs.identifier;
  elements   = rhs.elements;
  separator  = rhs.separator;

  return *this;
}


Definition&
Definition::
operator=(Definition&&  rhs)
{
  identifier = std::move(rhs.identifier);
  elements   = std::move(rhs.elements  );
  separator  = std::move(rhs.separator );

  return *this;
}


void
Definition::
reset(const char*&  p)
{
  clear();

  char  buf[256];

  int  n = 0;

    if(sscanf(p," %256[0-9a-zA-Z_] = %n",buf,&n) == 1)
    {
      identifier = buf;

      p += n;

      scan(p);
    }

  else
    {
      report;
    }
}


void
Definition::
clear()
{
  elements.clear();

  separator = 0;
}


void
Definition::
append_element(Element&&  el)
{
  elements.emplace_back(std::move(el));
}


const ElementList&
Definition::
get_list() const
{
  return elements;
}




void
Definition::
change_identifier(const char*  s)
{
  identifier = s;
}


const Identifier&
Definition::
get_identifier() const
{
  return identifier;
}




void
Definition::
change_separator(int  sep)
{
  separator = sep;
}


int
Definition::
get_separator() const
{
  return separator;
}




void
Definition::
print(Printer&  pr) const
{
  pr.printf("[%s] = ",identifier.get_buffer());

    if(elements.size())
    {
      auto   it = elements.cbegin();
      auto  end = elements.cend();

        for(;;)
        {
          it->print(pr);

            if(++it != end)
            {
              pr.putc(separator,true);
            }

           else
            {
              break;
            }
        }
    }


  pr.putc(';',true);
}


}




